/* eslint-disable max-len */
import {
  CATEGORY_LIST_API,
  PRODUCT_LIST_API,
  GET_PRODUCT_API,
  GET_CONFIG_PRODUCT_API,
  PRODUCTION_PUBLISH_STATE,
  ADD_PRODUCT_API,
  GET_ORDER_PRODUCT_API,
  GET_ORDER_PRODUCT_CATALOG_PROFILE_API,
  KEYWORD_SEARCH_API,
  QUICK_ORDER_PRODUCTS,
  IMPORT_FILE_API,
} from "config/constant";
import { httpApi } from "api/http.api";
import {
  IAddProductProp,
  IConfigurableProductModel,
  IKeywordSearchRequest,
  IQuickOrderParameter,
  ISearchRequest,
} from "interfaces/APIInterface/IProducts";
import { IProductSetLineItems } from "interfaces/ClientInterface/IEditOrder";

/**
 * This function sends a GET request to get the list of categories for the catalogId.
 */
export const getCategoryList = (catalogId: number, localeId: number, portalId: number) => {
  return httpApi
    .get(CATEGORY_LIST_API, {
      params: {
        expand: "",
        filter:
          "ZnodeCatalogId~eq~" +
          catalogId +
          ",LocaleId~eq~" +
          localeId +
          ",PortalId~eq~" +
          portalId +
          ",IsActive~eq~true,ZnodeParentCategoryIds~eq~",
        sort: "name~asc",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to get the list of products on some parameters
 */
export const getProductList = (searchRequest: ISearchRequest) => {
  return httpApi
    .post(PRODUCT_LIST_API, searchRequest, {
      params: {
        expand: "pricing",
        filter:
          "ZnodeCatalogId~eq~" +
          searchRequest.CatalogId +
          ",LocaleId~eq~" +
          searchRequest.LocaleId +
          ",PortalId~eq~" +
          searchRequest.PortalId +
          ",IsActive~eq~true,ZnodeCategoryId~eq~" +
          searchRequest.CategoryId +
          ",Enable_CMS_Page_Results~eq~True",
        sort: searchRequest.ColumnName + "~" + searchRequest.OrderType,
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to get the list of products on some parameters
 */
export const keywordSearchSuggestion = (searchRequest: IKeywordSearchRequest) => {
  return httpApi
    .post(KEYWORD_SEARCH_API, searchRequest, {
      params: {
        expand: "",
        filter:
          "IsActive~eq~True" +
          ",ProductIndex~eq~1" +
          ",ZnodeCatalogId~eq~" +
          searchRequest.CatalogId +
          ",LocaleId~eq~" +
          searchRequest.LocaleId +
          ",PortalId~eq~" +
          searchRequest.PortalId,
        sort: "",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to get the list of products for Quick Order
 */
export const getQuickOrderProductList = (quickOrderParameter: IQuickOrderParameter
  , catalogId: number, localeId: number, portalId: number) => {
  return httpApi
    .post(QUICK_ORDER_PRODUCTS, quickOrderParameter, {
      params: {
        expand: "",
        filter:
        "ZnodeCatalogId~eq~" +
        catalogId +
        ",LocaleId~eq~" +
        localeId +
        ",PortalId~eq~" +
        portalId + 
        ",IsActive~eq~True",
        sort: "",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};


/**
 * This function sends a GET request to get the specific product details
 */
export const getProductDetails = (
  publishProductId: number,
  catalogId: number,
  localeId: number,
  portalId: number,
  childSKU?: string
) => {
  return httpApi
    .get(`${GET_PRODUCT_API}/${publishProductId}`, {
      params: {
        expand: "inventory,pricing",
        filter:
          "ZnodeCatalogId~eq~" +
          catalogId +
          ",LocaleId~eq~" +
          localeId +
          ",PortalId~eq~" +
          portalId +
          ",ChildSku~eq~" +
          childSKU +
          ",IsActive~eq~true,ZnodeCategoryIds~ne~0",
        sort: "",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to get the specific configurable product details
 */
export const getConfigurableProductDetails = (productAttributes: IConfigurableProductModel) => {
  return httpApi
    .post(GET_CONFIG_PRODUCT_API, productAttributes, {
      params: {
        expand: "inventory,pricing",
        filter: "",
        sort: "",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to add the product to the order
 */
export const addProduct = (productModel: IAddProductProp) => {
  return httpApi
    .post(ADD_PRODUCT_API, { ...productModel })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a GET request to get the specific product details added to the order.
 */
export const getProductDetailsForEdit = (lineItemId: string | undefined) => {
  return httpApi
    .get(`${GET_ORDER_PRODUCT_API}/${lineItemId}`)
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a GET request to get the order product catalog and user profile details
 */
export const getOrderProductCatalogProfileDetails = (userId: number, portalId: number) => {
  return httpApi
    .get(`${GET_ORDER_PRODUCT_CATALOG_PROFILE_API}/${userId}/${portalId}`, {
      params: {
        expand: "",
        filter: "",
        sort: "",
        page: "",
        publishState: PRODUCTION_PUBLISH_STATE,
      },
    })
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};

export const SendCartLineItems = (requestlineItemDetails: IProductSetLineItems) => {
  return httpApi
    .post(
      "/product/sendlineitems" +
        "/" +
        requestlineItemDetails.orderId +
        "/" +
        requestlineItemDetails.lineItemIds
    )
    .then((response) => response)
    .catch((error) => {
      throw error;
    });
};

/**
 * This function sends a POST request to import the file into quick order cart
 */
export const parseFileIntoQuickOrderCartArray = (fileName: FormData) => {
  return httpApi
    .post(IMPORT_FILE_API,  fileName,
    )
    .then((response) => response?.data)
    .catch((error) => {
      throw error;
    });
};
