/* eslint-disable max-lines-per-function */
import React, { useEffect } from "react";
import { Col, Row, Typography, Input, Select, Form } from "antd";
import { AddAddressModalProps } from "interfaces/ClientInterface/IAddAddress";
import Checkbox from "antd/es/checkbox";
import { COUNTRIES, STATES } from "./AddAddress.Constant";
import ModalComponent from "../Modal/Modal";
import "../../Common/DuplicateModal/DuplicateModal.scss";
import { translate } from "i18n";
import { isPostalCodeValid, isValidPhoneNumber } from "utils/validation";
const { Text } = Typography;

/**
 * @returns React functional component that renders a form for adding a new address. It uses
 * the useState hook to manage the state of the form inputs.
 */
const AddAddressModal = ({
  title,
  isAddressModalVisible,
  onAddressSave,
  addressDetails,
  setAddressDetails,
  onModalCancelPress,
}: AddAddressModalProps) => {
  const [form] = Form.useForm();

  useEffect(() => {
    form.setFieldsValue({
      ...addressDetails,
    });
  }, [addressDetails, form]);

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const onOkHandler = (values: any) => {
    const body = {
      ...values,
    };

    setAddressDetails(body);
    onAddressSave && onAddressSave(body, form);
  };

  return (
    <ModalComponent
      title={title}
      visible={isAddressModalVisible}
      onCancel={() => {
        onModalCancelPress(form);
      }}
      onOK={() => {
        form.validateFields().then((values) => {
          if (!values?.errorFields) {
            onOkHandler(values);
          }
        });
      }}
      okText="Save"
      cancelText="Cancel"
      bodyStyle={{ overflowY: "auto", maxHeight: "calc(100vh - 200px)" }}
      dataTestSelector="popAddShippingAddress"
      className="standard-modal"
    >
      <Form form={form} initialValues={addressDetails} scrollToFirstError autoComplete={"false"}>
        <Row className="mt-16" gutter={[0, 20]}>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleAddressName")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"DisplayName"}
              className="m-0"
              rules={[
                { required: true, message: translate("addAddress.errorAddressNameRequired") },
              ]}
            >
              <Input data-test-selector="txtAddressName" autoComplete="txtAddressName" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleFirstName")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"FirstName"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorFirstNameRequired") }]}
            >
              <Input data-test-selector="txtFirstName" autoComplete="txtFirstName" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleLastName")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"LastName"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorLastNameRequired") }]}
            >
              <Input data-test-selector="txtLastName" autoComplete="txtLastName" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title">{translate("addAddress.titleCompanyName")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"CompanyName"}
              className="m-0"
              rules={[{ message: translate("addAddress.errorCompanyNameRequired") }]}
            >
              <Input data-test-selector="txtCompanyName" autoComplete="txtCompanyName" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleAddressLine1")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"Address1"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorAddress1Required") }]}
            >
              <Input data-test-selector="txtAddress1" autoComplete="txtAddress1" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title">{translate("addAddress.titleAddressLine2")}:</Text>
          </Col>
          <Col span={14}>
            <Form.Item name={"Address2"} className="m-0">
              <Input data-test-selector="txtAddress2" autoComplete="txtAddress2" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title">{translate("addAddress.titleCountry")}:</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"CountryName"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorCountryRequired") }]}
            >
              <Select
                className="w-100"
                showSearch
                options={COUNTRIES}
                placeholder={translate("addAddress.placeholderSelectCountry")}
                allowClear
                data-test-selector="drpCountry"
                autoFocus
              />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleState")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"StateName"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorStateRequired") }]}
            >
              <Select
                className="w-100"
                showSearch
                options={STATES}
                placeholder={translate("addAddress.titleSelectState")}
                allowClear
                data-test-selector="drpState"
                fieldNames={{ label: "Text", value: "Value" }}
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                filterOption={(input, option?: any) =>
                  option?.Text?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
                }
                autoFocus
              />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titleCity")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"City"}
              className="m-0"
              rules={[{ required: true, message: translate("addAddress.errorCityRequired") }]}
            >
              <Input data-test-selector="txtCity" autoComplete="txtCity" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titlePostalCode")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"PostalCode"}
              className="m-0"
              rules={[
                () => ({
                  validator(_, value) {
                    if (!value) {
                      return Promise.reject(
                        new Error(translate("addAddress.errorPostalCodeRequired"))
                      );
                    }
                    if (value.trim() !== "" && !isPostalCodeValid(value)) {
                      return Promise.reject(new Error(translate("addAddress.invalidPostalCode")));
                    }
                    return Promise.resolve();
                  },
                }),
              ]}
            >
              <Input data-test-selector="txtPostalCode" autoComplete="txtPostalCode" />
            </Form.Item>
          </Col>
          <Col span={10}>
            <Text className="key-title required">{translate("addAddress.titlePhoneNumber")}</Text>
          </Col>
          <Col span={14}>
            <Form.Item
              name={"PhoneNumber"}
              className="m-0"
              rules={[
                { required: true, message: translate("addAddress.errorPhoneNumberRequired") },
                {
                  validator: (_, value) => {
                    if (value && value.length > 20) {
                      return Promise.reject(
                        new Error(translate("addAddress.errorPhoneNumberValid"))
                      );
                    }
                    if (value && !isValidPhoneNumber.test(value)) {
                      return Promise.reject(
                        new Error(translate("addAddress.errorPhoneNumberValid"))
                      );
                    }
                    return Promise.resolve();
                  },
                },
              ]}
            >
              <Input data-test-selector="txtPhoneNumber" autoComplete="txtPhoneNumber" />
            </Form.Item>
          </Col>
          <Col span={14} push={10}>
            <Form.Item valuePropName="checked" name={"IsDefaultBilling"} className="m-0">
              <Checkbox data-test-selector={"chkDefaultBilling"}>
                {translate("addAddress.checkboxLabelDefaultBillingAddress")}
              </Checkbox>
            </Form.Item>
          </Col>
          <Col span={14} push={10}>
            <Form.Item valuePropName="checked" name={"IsDefaultShipping"} className="m-0">
              <Checkbox data-test-selector={"chkDefaultShipping"}>
                {translate("addAddress.checkboxLabelShippingAddress")}
              </Checkbox>
            </Form.Item>
          </Col>
        </Row>
      </Form>
    </ModalComponent>
  );
};

export default AddAddressModal;
