import React, { useContext, useState } from "react";
import { Avatar, Space } from "antd";
import { CloseCircleOutlined } from "@ant-design/icons";
import TagComponent from "components/Common/Tag/Tag";

import "./AssignedTo.scss";
import { AssignedToProps, IAssignToListResponse } from "interfaces/ClientInterface/IAssignedTo";
import { IUserGroup } from "interfaces/APIInterface/IUserGroup";
import { updateOrderOwner, deleteOrderUserGroup } from "api/services/editOrder.api";
import { notificationController } from "../Notification/Notification";
import { readUserId } from "api/services/localStorage.service";
import AssignToPopOver from "./AssignToModal";
import { IAssignUnassign } from "interfaces/APIInterface/IAssignTo";
import { UsergroupAddOutlined } from "@ant-design/icons";
import { PureAbility } from "@casl/ability";
import { AbilityContext } from "ability/can";
import { translate } from "i18n";
import { useAppSelector } from "hooks/reduxHooks";
import { PERMISSION_CODE, PERMISSION_SUBJECT } from "ability/utils";

export default function AssignedTo({
  order,
  groups,
  setOrder,
  setOrderUserGroup,
  notesChildRef,
  getOrderDetails,
}: AssignedToProps) {
  const ability: PureAbility<[string, string]> = useContext(AbilityContext);
  const isEditPermission = useAppSelector((state) => state.permission.isEditPermission);
  const unAssignPermission = ability.can(
    PERMISSION_CODE.UNASSIGN_ORDERS,
    PERMISSION_SUBJECT.ORDERS
  );
  const [open, setOpen] = useState(false);
  const owner = order?.AssignToFullName;
  const orderId = order?.OrderId;
  const onDeleteOrderUserGroup = (orderUserGroupId: string) => {
    deleteOrderUserGroup(orderUserGroupId)
      .then((response) => {
        if (response.IsSuccess) {
          notificationController.success({
            message: translate("assignedTo.userGroupRemovedSuccessMessage"),
          });
          const groupData = groups;
          const filteredUserGroup = groupData.filter((item) => {
            return item?.OrderUserGroupId != orderUserGroupId;
          });
          setOrderUserGroup(filteredUserGroup || []);
        }
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  const onUnassignOrderOwner = (orderId: string) => {
    const assignUnassignPayload: IAssignUnassign = {
      OrderIds: orderId,
      UserId: readUserId(),
      AssignTo: 0,
    };
    updateOrderOwner(assignUnassignPayload)
      .then(() => {
        const orderData = { ...order };
        orderData.AssignToFullName = "";
        setOrder(orderData);
        notificationController.success({
          message: translate("assignedTo.ownerRemovedSuccessMessage"),
        });
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  const updateDataAfterSave = (assignToResponse: IAssignToListResponse, userGroup: string) => {
    if (assignToResponse?.AssignToFullName) {
      const orderData = { ...order };
      orderData.AssignToFullName = assignToResponse?.AssignToFullName;
      setOrder(orderData);
    }
    if (userGroup) {
      setOrderUserGroup(assignToResponse?.UserGroupsList?.UserGroupsList || []);
    }
  };

  return (
    <Space size={1} direction="vertical" className="mt-10">
      <div className="d-flex">
        <AssignToPopOver
          popOverTitle={`${translate("editOrder.assignedTo")}:`}
          open={open}
          setOpen={setOpen}
          icon={
            <Avatar
              style={{ backgroundColor: "#07389B" }}
              size={20}
              icon={<UsergroupAddOutlined />}
            />
          }
          updateDataAfterSave={updateDataAfterSave}
          isEditPermission={isEditPermission}
          notesChildRef={notesChildRef}
          getOrderDetails={getOrderDetails}
        />
      </div>
      <div className="assigned-sub-title d-flex align-center">
        {translate("assignedTo.titleOwner")}:
        {owner ? (
          <Space size={5}>
            <TagComponent
              key={orderId}
              id={orderId}
              text={owner}
              closeIcon={<CloseCircleOutlined className="tag-close-icon" />}
              tagContainerClass="tag-container-transparent"
              tagContentClass="tag-content-transparent"
              tagColor="transparent"
              closable={
                isEditPermission
                  ? owner == "Unassigned" || !unAssignPermission
                    ? false
                    : true
                  : false
              }
              dataTestSelector="tagOwner"
              onCloseClick={(id: string) => {
                onUnassignOrderOwner(id);
                notesChildRef;
                getOrderDetails();
              }}
            />
          </Space>
        ) : (
          <span className="unassigned-title">{translate("assignedTo.titleUnassigned")}</span>
        )}
      </div>
      <div className="assigned-sub-title d-flex align-start">
        {translate("assignedTo.titleGroups")}:
        <Space size={0} className="flex-wrap">
          {groups &&
            groups?.length > 0 &&
            groups?.map((item: IUserGroup) => {
              return (
                <TagComponent
                  key={item?.OrderUserGroupId}
                  id={item?.OrderUserGroupId}
                  text={item?.GroupName}
                  closeIcon={<CloseCircleOutlined className="tag-close-icon" />}
                  tagContainerClass="tag-container-transparent"
                  tagContentClass="tag-content-transparent"
                  tagColor="transparent"
                  closable={isEditPermission}
                  dataTestSelector="tagGroup"
                  onCloseClick={(id: string) => {
                    onDeleteOrderUserGroup(id);
                    notesChildRef;
                    getOrderDetails();
                  }}
                />
              );
            })}
        </Space>
      </div>
    </Space>
  );
}
