/* eslint-disable max-lines-per-function */
import React, { useContext, useEffect, useState } from "react";
import { Space, Button, Row, Col, Select, Spin } from "antd";
import { TitleHeader } from "components/Common/TitleHeader/TitleHeader";
import { Typography } from "antd";
import { IStoreList, ICreateOrder } from "interfaces/APIInterface/ICreateOrder";
import { ORDER_CLASS, ORDER_ORIGIN, TYPEAHEAD_TYPE, TYPEAHEAD_TYPE_NAME } from "config/constant";
import { notificationController } from "components/Common/Notification/Notification";
import { ArrowLeftOutlined } from "@ant-design/icons";
import {
  IAccountListItem,
  IOrderTypeListItem,
  IStoreListItem,
  IUserListItem,
} from "interfaces/APIInterface/ICreateOrder";
import {
  getOrderTypeList,
  getAccountList,
  getUserList,
  getStoreList,
  createOrder,
} from "api/services/createOrder.api";
import { useNavigate } from "react-router-dom";
import { readUserId } from "api/services/localStorage.service";
import "./CreateOrder.scss";
import ModalComponent from "components/Common/Modal/Modal";
import { ROUTES_PATH } from "routes/Routes.Constant";
import { translate } from "i18n";
import "../Common/DuplicateModal/DuplicateModal.scss";
import { AbilityContext } from "ability/can";
import { NoAccessPage } from "components/Common/NoAccess/NoAccess";
import { PERMISSION_CODE, PERMISSION_SUBJECT } from "ability/utils";

const { Text } = Typography;

/**
 * The CreateOrder function is a React component that renders
 * a form for creating an order.
 * @returns It renders a form with several input fields and buttons.
 * When the "Save & Edit" button is clicked,
 * it calls the createOrder function which sends
 * a POST request to an API endpoint using httpApi.
 */
const CreateOrder = () => {
  const ability = useContext(AbilityContext);
  const navigate = useNavigate();
  const [store, setStore] = useState<string>("");
  const [account, setAccount] = useState<string>("");
  const [user, setUser] = useState<string>("");
  const [orderType, setOrderType] = useState<string>("");
  const [searchTerm, setSearchTerm] = useState<string>("");
  const [isConfirmModalVisible, setIsConfirmModalVisible] = useState<boolean>(false);

  const [loader, setLoader] = useState({
    storeLoading: false,
    accountLoading: false,
    userLoading: false,
    orderTypeLoading: false,
  });

  const [error, setError] = useState({
    storeError: "",
    userError: "",
    orderTypeError: "",
  });

  //Lists
  const [orderTypeList, setOrderTypeList] = useState<IOrderTypeListItem[]>([]);
  const [accountList, setAccountList] = useState<IAccountListItem[]>([]);
  const [userList, setUserList] = useState<IUserListItem[]>([]);
  const [storeList, setStoreList] = useState<IStoreListItem[]>([]);

  useEffect(() => {
    if (searchTerm) {
      searchTerm.length >= 3 && fetchUserList();
    } else {
      setUserList([]);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [searchTerm]);

  // This is to reset user, account and searchTerm when the selected store is changed.
  useEffect(() => {
    setUser("");
    setAccount("");
    setSearchTerm("");
    setAccountList([]);
    setUserList([]);
  }, [store]);

  // This is to reset user list and search term when the user is not selected.
  useEffect(() => {
    if (!user) {
      setSearchTerm("");
      setUserList([]);
    }
  }, [user]);

  /**
   * The function creates an order by sending a POST request with a payload to an API endpoint using
   * httpApi.
   */
  const onCreateOrder = (action: string) => {
    // This function validates store dropdown.
    const validateStore = () => {
      if (!store) {
        setError((prevState) => ({
          ...prevState,
          storeError: translate("createOrder.storeRequiredErrorMessage"),
        }));
        return false;
      }
      return true;
    };

    // This function validates user dropdown.
    const validateUser = () => {
      if (store) {
        if (!user) {
          setError((prevState) => ({
            ...prevState,
            userError: translate("createOrder.userRequiredErrorMessage"),
          }));
          return false;
        }
      }

      return true;
    };

    // This function validates order type dropdown.
    const validateOrderType = () => {
      if (!orderType) {
        setError((prevState) => ({
          ...prevState,
          orderTypeError: translate("createOrder.orderTypeRequiredErrorMessage"),
        }));
        return false;
      }
      return true;
    };

    const isValidStore = validateStore();
    const isValidUser = validateUser();
    const isValidOrderType = validateOrderType();

    if (!isValidStore || !isValidUser || !isValidOrderType) {
      return;
    }

    //To get the order type details of selected order type.
    const orderTypeDetails = orderTypeList?.find((item: IOrderTypeListItem) => {
      return item?.CPOrderTypeId?.toString() == orderType;
    });

    const createOrderPayload: ICreateOrder = {
      PortalId: store,
      UserId:  readUserId(),
      AccountId: account,
      OrderTypeCode: orderTypeDetails?.TypeCode || "",
      OrderDate: new Date(),
      OrderOrigin: ORDER_ORIGIN,
      ClassCode: ORDER_CLASS,
      AssignTo: user,
    };
    createOrder(createOrderPayload)
      .then((res) => {
        const orderId = res?.data?.StatusModel?.Id;
        if (action === "edit") {
          navigate(`/${ROUTES_PATH.EDIT_ORDER}/${orderId}`, {
            state: {
              cameFromCreateOrder: true,
            },
          });
        } else {
          navigate(-1);
        }
        notificationController.success({
          message: translate("createOrder.recordCreatedSuccessMessage"),
          placement: "top",
        });
      })
      .catch(() => {
        notificationController.error({ message: translate("createOrder.orderFailedErrorMessage") });
      });
  };

  const setLoading = (key: string, loaderState: boolean) => {
    setLoader((prevState) => ({
      ...prevState,
      [key]: loaderState,
    }));
  };

  /**
   * This function sends a Get request to get the list of order types.
   */
  const fetchOrderTypeList = () => {
    setLoading("orderTypeLoading", true);
    getOrderTypeList()
      .then((response) => {
        const data = response?.data?.OrderType;
        setOrderTypeList(data);
        setLoading("orderTypeLoading", false);
      })
      .catch(() => {
        setLoading("orderTypeLoading", false);
      });
  };

  /**
   * This function sends a GET request to get the list of accounts.
   */
  const fetchAccountList = () => {
    // This is to get the account id of the selected user.
    const UserDetails = userList?.find((item: IUserListItem) => {
      return item?.UserId?.toString() == user;
    });
    setLoading("accountLoading", true);
    getAccountList(store, UserDetails?.AccountId || "")
      .then((response) => {
        const data = response?.data?.Accounts;
        setAccountList(data);
        if (user) {
          if (UserDetails?.AccountId === null || undefined) {
            setAccountList([]);
          }
        }
        setLoading("accountLoading", false);
      })
      .catch(() => {
        setLoading("accountLoading", false);
      });
  };

  /**
   * This function sends a GET request to get the list of users.
   */
  const fetchUserList = () => {
    setLoading("userLoading", true);
    getUserList(store, searchTerm, account)
      .then((response) => {
        const data = response?.data?.Users;
        setUserList(data);
        setLoading("userLoading", false);
      })
      .catch(() => {
        setLoading("userLoading", false);
      });
  };

  /**
   * This function sends a GET request to get the list of stores.
   */
  const fetchStoreList = () => {
    const typeaheadRequestPayload: IStoreList = {
      MappingId: 0,
      PageSize: 0,
      FieldName: "",
      Type: TYPEAHEAD_TYPE,
      TypeName: TYPEAHEAD_TYPE_NAME,
      Searchterm: "",
    };
    setLoading("storeLoading", true);
    getStoreList(typeaheadRequestPayload)
      .then((response) => {
        const data = response?.data?.Typeaheadlist;
        setStoreList(data);
        setLoading("storeLoading", false);
      })
      .catch(() => {
        setLoading("storeLoading", false);
      });
  };

  const saveAndCloseHandler = () => {
    onCreateOrder("close");
  };

  const saveAndEditHandler = () => {
    onCreateOrder("edit");
  };

  const checkIfAnyChangesDone = () => {
    if (store != "" || account != "" || user != "" || orderType != "") {
      setIsConfirmModalVisible(true);
    } else {
      setIsConfirmModalVisible(false);
      navigate(-1);
    }
  };

  if (!ability.can(PERMISSION_CODE.CREATE_ORDERS, PERMISSION_SUBJECT.ORDERS)) {
    return <NoAccessPage />;
  }

  return (
    <Space direction="vertical" size="middle" className="d-flex">
      <div className="title-header-wrapper">
        <TitleHeader
          title={translate("createOrder.title")}
          isBack={true}
          backClickHandler={checkIfAnyChangesDone}
          rhsContent={
            <Space size={10}>
              <Button
                type="primary"
                data-test-selector={"btnSaveAndEdit"}
                onClick={saveAndEditHandler}
              >
                {translate("createOrder.saveAndEditBtn")}
              </Button>
              <Button
                type="primary"
                data-test-selector={"btnSaveAndClose"}
                onClick={saveAndCloseHandler}
              >
                {translate("createOrder.saveAndCloseBtn")}
              </Button>
              <Button
                type="primary"
                ghost
                data-test-selector={"btnCancel"}
                onClick={checkIfAnyChangesDone}
              >
                {translate("createOrder.cancelBtn")}
              </Button>
            </Space>
          }
        />
      </div>
      <Row gutter={[0, 40]} className="order-form-container">
        <Col lg={10} md={10} sm={24} xs={24}>
          <Row justify={"start"} align={"middle"} className="flex-baseline">
            <Col md={6} sm={6}>
              <Text className="select-menu-title w-100 required">
                {translate("createOrder.storeLabel")}
              </Text>
            </Col>
            <Col md={14} sm={18} xs={24}>
              <Select
                className="w-100"
                showSearch
                options={storeList}
                placeholder={translate("createOrder.selectStorePlaceholder")}
                allowClear
                value={store || undefined}
                onChange={(store: string) => {
                  setStore(store);
                  setError((prevState) => ({
                    ...prevState,
                    storeError: "",
                  }));
                }}
                onClick={fetchStoreList}
                data-test-selector="drpStoreList"
                fieldNames={{ label: "Name", value: "Id" }}
                loading={loader?.storeLoading}
                // eslint-disable-next-line
                filterOption={(input, option?: any) =>
                  option?.Name?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
                }
                notFoundContent={null}
              />
              <span className="error">{error && error?.storeError}</span>
            </Col>
          </Row>
        </Col>
        <Col lg={10} md={10} sm={24} xs={24}>
          <Row justify={"start"} align={"middle"} className="flex-baseline">
            <Col md={6} sm={6}>
              <Text className="select-menu-title">{translate("createOrder.accountLabel")}</Text>
            </Col>
            <Col md={14} sm={18} xs={24}>
              <Select
                className="w-100"
                showSearch
                options={accountList}
                placeholder={translate("createOrder.selectAccountPlaceholder")}
                allowClear
                value={account || undefined}
                onChange={(account: string) => setAccount(account)}
                onClick={fetchAccountList}
                data-test-selector="drpAccountsList"
                fieldNames={{ label: "Name", value: "AccountId" }}
                loading={loader?.accountLoading}
                disabled={!store}
                // eslint-disable-next-line
                filterOption={(input, option?: any) =>
                  option?.Name?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
                }
                notFoundContent={null}
              />
            </Col>
          </Row>
        </Col>
        <Col lg={10} md={10} sm={24} xs={24}>
          <Row justify={"start"} align={"middle"} className="flex-baseline">
            <Col md={6} sm={6}>
              <Text className="select-menu-title required">
                {translate("createOrder.orderTypeLabel")}
              </Text>
            </Col>
            <Col md={14} sm={18} xs={24}>
              <Select
                className="w-100"
                showSearch
                options={orderTypeList}
                placeholder={translate("createOrder.selectOrderTypePlaceholder")}
                allowClear
                value={orderType || undefined}
                onChange={(orderType: string) => {
                  setOrderType(orderType);
                  setError((prevState) => ({
                    ...prevState,
                    orderTypeError: "",
                  }));
                }}
                onClick={fetchOrderTypeList}
                data-test-selector="drpOrderTypeList"
                fieldNames={{ label: "TypeName", value: "CPOrderTypeId" }}
                loading={loader?.orderTypeLoading}
                // eslint-disable-next-line
                filterOption={(input, option?: any) =>
                  option?.TypeName?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
                }
                notFoundContent={null}
              />
              <span className="error">{error && error?.orderTypeError}</span>
            </Col>
          </Row>
        </Col>
        <Col lg={10} md={10} sm={24} xs={24}>
          <Row justify={"start"} align={"middle"} className="flex-baseline">
            <Col md={6} sm={6}>
              <Text className="select-menu-title required">
                {translate("createOrder.userLabel")}
              </Text>
            </Col>
            <Col md={14} sm={18} xs={24}>
              <Select
                className="w-100"
                showSearch
                options={userList}
                placeholder={translate("createOrder.searchByPlaceholder")}
                allowClear
                value={user || undefined}
                onChange={(user: string) => {
                  setUser(user);
                  setError((prevState) => ({
                    ...prevState,
                    userError: "",
                  }));
                }}
                notFoundContent={loader?.userLoading ? <Spin size="small" /> : null}
                filterOption={false}
                onSearch={(searchTerm: string) => setSearchTerm(searchTerm)}
                data-test-selector="drpUserList"
                fieldNames={{ label: "UserName", value: "UserId" }}
                loading={loader?.userLoading}
                disabled={!store}
              />
              <span className="error">{store && error && error?.userError}</span>
            </Col>
          </Row>
        </Col>
      </Row>
      <ModalComponent
        title={translate("createOrder.pleaseConfirmTitle")}
        visible={isConfirmModalVisible}
        onOK={() => {
          setIsConfirmModalVisible(false);
          navigate(-1);
        }}
        onCancel={() => {
          setIsConfirmModalVisible(false);
        }}
        okText="OK"
        cancelText={
          <Space className="d-flex justify-center align-center">
            <ArrowLeftOutlined />
            <span>{translate("createOrder.backBtn")}</span>
          </Space>
        }
        dataTestSelector="popConfirm"
        zIndex={10}
        isMaskClosable={true}
        className="standard-modal"
      >
        {translate("createOrder.confirmModalMessage")}
      </ModalComponent>
    </Space>
  );
};

export default CreateOrder;
