import React, { useState, useEffect } from "react";
import { Row, Col, Space, Button } from "antd";
import { Content } from "antd/es/layout/layout";
import Metrics from "../Common/Metrics/Metrics";
import CardComponent from "components/Common/Card/Card";
import List from "components/Common/List/List";
import { columns } from "./Dashboard.Constant";
import { notificationController } from "components/Common/Notification/Notification";
import { IMetricsItemsModel } from "interfaces/APIInterface/IDashboard";
import { IDuplicateOrder, IOrderListData } from "interfaces/ClientInterface/IOrderList";
import { duplicateOrder, getOrderList } from "api/services/manageOrder.api";
import { useNavigate } from "react-router-dom";
import { IQuickViewOpen } from "interfaces/ClientInterface/IQuickViewOrder";
import DuplicateModalComponent from "components/Common/DuplicateModal/DuplicateModal";
import { ArrowLeftOutlined } from "@ant-design/icons";
import { ROUTES_PATH } from "routes/Routes.Constant";
import { IOrderDetails } from "interfaces/APIInterface/IQuickViewOrder";
import { getQuickViewData } from "components/Orders/Order.Constant";
import { IDuplicateOrderPayload } from "interfaces/APIInterface/IOrderList";
import { translate } from "i18n";
import { getMetricsData, getQuickViewDetails } from "api/services/dashboard.api";
import "./Dashboard.scss";
import { IData } from "interfaces/ClientInterface/IList";

// eslint-disable-next-line max-lines-per-function
export default function Dashboard() {
  const navigate = useNavigate();
  const [orders, setOrders] = useState<IData[]>([]);
  // eslint-disable-next-line no-unused-vars
  const [orderListParams, setOrderListParams] = useState({
    portalId: -1,
    classNumber: "",
    category: "",
    account: "",
    userFullName: "",
    orderStateName: "",
    orderTotal: "",
    globalSearch: "",
    operatorClassNumber: "",
    operatorAccountName: "",
    operatorUserName: "",
    operatorOrderStateName: "",
    operatorOrderTotal: "",
    assignToFullName: "",
    operatorAssignToFullName: "",
    operatormodifiedDate: "",
    modifiedDate: "",
    operatorcreatedDate: "",
    createdDate: "",
    columnSorting: "",
  } as IOrderListData);

  const [quickViewData, setQuickViewData] = useState<IOrderDetails>({} as IOrderDetails);
  const [quickViewLoading, setQuickViewLoading] = useState<boolean>(false);

  const [isDuplicateOrder, setIsDuplicateOrder] = useState<IDuplicateOrder>({
    visible: false,
    orderId: "",
  });
  //Added the const to get metrics details from the interface.
  const [metricsData, setMetricsData] = useState<IMetricsItemsModel>({
    TotalOrders: "",
    TotalQuotes: "",
    TotalReturns: "",
    TotalSales: "",
  });
  const [quickViewOpen, setQuickViewOpen] = useState<IQuickViewOpen>({
    id: "",
    visible: false,
  });
  const [sortingColumn, setSortingColumn] = useState({
    sortByColumnName: "",
    sortType: "",
  });
  const onEditItemPress = (data: string) => {
    navigate(`/${ROUTES_PATH.EDIT_ORDER}/${data}`);
  };

  useEffect(() => {
    fetchOrderList();
    fetchMetrics();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);

  const onQuickView = (recordId: string) => {
    setQuickViewLoading(true);
    getQuickViewDetails(recordId)
      .then((response) => {
        const data = response?.Order;
        const quickViewDetails = getQuickViewData(data);
        setQuickViewData(quickViewDetails);
        setQuickViewLoading(false);
      })
      .catch((err) => {
        setQuickViewLoading(false);
        notificationController.error({ message: err.message });
      });
    setTimeout(() => {
      setQuickViewOpen({ id: recordId, visible: true });
    });
  };

  const fetchMetrics = () => {
    getMetricsData()
      .then((response) => {
        const data: IMetricsItemsModel = response?.MetricsItemsModel?.[0];
        setMetricsData(data);
      })
      .catch((err) => {
        notificationController.error({ message: err.message, placement: "top" });
      });
  };
  /**
   * Get the Order list in dashboard page
   */
  const fetchOrderList = () => {
    getOrderList(orderListParams)
      .then((response) => {
        const dataList: IData[] = response?.data?.OrderList;
        const filterDataList = dataList?.filter((item: IData, i: number) => {
          return i <= 9;
        });
        setOrders(filterDataList);
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  const duplicateRecord = (withShipping: boolean) => {
    const requestBody: IDuplicateOrderPayload = {
      OrderId: isDuplicateOrder?.orderId,
      IsCopyOrderShipment: withShipping,
    };
    duplicateOrder(requestBody)
      .then((response) => {
        const data = response?.data;
        if (data?.IsSuccess) {
          notificationController.success({
            message: translate("dashboard.recordDuplicateSuccessMessage"),
          });
          fetchOrderList();
          handleBack();
        }
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  const handleBack = () => {
    setIsDuplicateOrder({
      visible: false,
      orderId: "",
    });
  };

  const handleWithoutShipping = () => {
    duplicateRecord(false);
  };

  const handleWithShipping = () => {
    duplicateRecord(true);
  };

  const quickViewProps = {
    quickViewOpen,
    setQuickViewOpen,
    quickViewData,
    onQuickView,
    quickViewLoading,
  };

  const cols = columns(onEditItemPress, quickViewProps, setIsDuplicateOrder);

  return (
    <>
      <Row className="card-container">
        <Col span={16}>
          <Content className="listing-container">
            <Space direction="vertical" size="middle" className="d-flex">
              <CardComponent
                cardTitle={translate("dashboard.ordersCardTitle")}
                addButtonLabel={translate("dashboard.addCreateOrder")}
                addButtonUrl={`/${ROUTES_PATH.CREATE_ORDER}`}
                addButtonDataTestSelector={"createOrder"}
                viewAllLabel={translate("dashboard.viewAllLabel")}
                viewAllUrl={`/${ROUTES_PATH.ORDERS}`}
                viewAllDataTestSelector={"viewAllOrder"}
              >
                <List
                  data={orders}
                  columns={cols}
                  pagination={false}
                  classSelector={"dashboard-listing-container"}
                  sortingColumn={sortingColumn}
                  setSortingColumn={setSortingColumn}
                />
              </CardComponent>
            </Space>
          </Content>
        </Col>
        <Col span={8}>
          <Content className="grids-container">
            <Space direction="vertical" size="middle" className="d-flex">
              <Metrics
                TotalOrder={metricsData?.TotalOrders}
                TotalQuote={metricsData?.TotalQuotes}
                TotalReturn={metricsData?.TotalReturns}
                TotalSale={metricsData?.TotalSales}
              />
            </Space>
          </Content>
        </Col>
        <Col span={24}>
          <DuplicateModalComponent
            title={translate("dashboard.confirmDuplicateMessage")}
            visible={isDuplicateOrder?.visible}
            onCancel={handleBack}
            dataTestSelector="popConfirm"
            zIndex={10}
            isMaskClosable={true}
            footer={null}
            className="duplicate-modal-popup"
          >
            {translate("dashboard.duplicateModalMessage")}
            <div className="popup-action mt-24">
              <Space>
                <Button icon={<ArrowLeftOutlined />} onClick={handleBack} size={"middle"}>
                  {translate("dashboard.backBtn")}
                </Button>
                <Button type="primary" onClick={handleWithoutShipping} size={"middle"}>
                  {translate("dashboard.withoutShippingInformationBtn")}
                </Button>
                <Button type="primary" onClick={handleWithShipping} size={"middle"}>
                  {translate("dashboard.withShippingInformationBtn")}
                </Button>
              </Space>
            </div>
          </DuplicateModalComponent>
        </Col>
      </Row>
    </>
  );
}
