import React from "react";
import { Col, Row, Space, Typography } from "antd";
import { CheckBoxComponent } from "components/Common/Checkbox/Checkbox";
import { CheckboxChangeEvent } from "antd/es/checkbox";
import { StopFilled } from "@ant-design/icons";
import {
  IAddUpdateOrderFlag,
  IOrderFlag,
  IOrderFlagProps,
} from "interfaces/ClientInterface/IOrderFlag";
import { deleteOrderFlag, updateOrderFlag } from "api/services/editOrder.api";
import { notificationController } from "components/Common/Notification/Notification";
import { readUserId } from "api/services/localStorage.service";
import { translate } from "i18n";
import { useAppSelector } from "hooks/reduxHooks";
const { Text } = Typography;

/**
 * @returns This React functional component returns a JSX element that renders
 * multiple checkboxes to check between different order flags
 */
const OrderFlagsContent: React.FC<IOrderFlagProps> = ({
  orderFlags,
  getUnAssociatedOrderFlag,
  getOrderDetailsWithFlag,
  notesChildRef,
  getOrderDetails
}: IOrderFlagProps) => {
  const isEditPermission = useAppSelector((state) => state.permission.isEditPermission);
  const onChangeOrderFlag = (isChecked: boolean, orderFlagId: string) => {
    const updateOrderFlagPayload: IAddUpdateOrderFlag = {
      UserId: readUserId(),
      OrderFlagId: orderFlagId,
      IsChecked: isChecked,
    };
    triggerUpdate(updateOrderFlagPayload);
  };

  const onDeleteOrderFlag = (orderFlagId: string) => {
    deleteOrderFlag(orderFlagId)
      .then((response) => {
        if (response.IsSuccess) {
          notificationController.success({
            message: translate("editOrder.flagRemovedSuccessMessage"),
          });
          getOrderDetailsWithFlag();
          getUnAssociatedOrderFlag && getUnAssociatedOrderFlag(false);
        }
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  const onChangeIsOverriddenActive = async (orderFlagId: string, isActive: boolean) => {
    const updateOrderFlagPayload: IAddUpdateOrderFlag = {
      UserId: readUserId(),
      OrderFlagId: orderFlagId,
      IsActiveOverridden: isActive,
      IsCallForOverrideUserGroup: true,
    };
    await triggerUpdate(updateOrderFlagPayload, true);
  };

  const triggerUpdate = (updateOrderFlagPayload: IAddUpdateOrderFlag, showMessage?: boolean) => {
    return updateOrderFlag(updateOrderFlagPayload)
      .then((response) => {
        if (response?.OrderFlagStatus?.IsSuccess && showMessage) {
          notificationController.success({
            message: translate("editOrder.flagStatusSuccessMessage"),
          });
        } else if (
          updateOrderFlagPayload.IsCallForOverrideUserGroup &&
          !response?.OrderFlagStatus?.IsSuccess &&
          !response?.OrderFlagStatus?.IsAllowFlagOverride
        ) {
          notificationController.error({
            message: translate("editOrder.loggedInUserNotPermittedToOverrideTheFlag"),
          });
        } else if (
          updateOrderFlagPayload.IsCallForOverrideUserGroup &&
          response?.OrderFlagStatus?.IsSuccess &&
          response?.OrderFlagStatus?.IsAllowFlagOverride
        ) {
          notificationController.success({
            message: translate("editOrder.flagStatusSuccessMessage"),
          });
        }
        getOrderDetailsWithFlag();
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  return (
    <Row justify={"start"} className="summary-container">
      <Col span={24}>
        <Space direction="horizontal" size={25} style={{ display: "flex", flexWrap: "wrap" }}>
          {orderFlags &&
            orderFlags?.map((item: IOrderFlag, i: number) => {
              return (
                <>
                  {item.IsActiveOverridden ? (
                    <Space size={10} key={i} id={item.OrderFlagId}>
                      <StopFilled
                        style={{ color: "#FF4D4F" }}
                        {...(isEditPermission
                          ? { onClick: () => onChangeIsOverriddenActive(item.OrderFlagId, false) }
                          : {})}
                      />
                      <Text>{item.NameOverridden}</Text>
                    </Space>
                  ) : (
                    <CheckBoxComponent
                      key={i}
                      id={item.OrderFlagId}
                      name={item.IsChecked ? item.NameChecked : item.NameUnchecked}
                      checked={item.IsChecked}
                      closable={isEditPermission}
                      dataTestSelector={"chkOrderFlag"}
                      onChangeHandle={(e: CheckboxChangeEvent) => {
                        onChangeOrderFlag(e.target.checked, item.OrderFlagId);
                        notesChildRef;
                        getOrderDetails();
                        notificationController.success({
                          message: translate("editOrder.flagStatusSuccessMessage"),
                        });
                      }}
                      onCancelHandle={(id: string) => {
                        onDeleteOrderFlag(id);
                        notesChildRef;
                        getOrderDetails();
                      }}
                      onOverriddenHandle={(id: string) => {
                        !item.IsChecked && onChangeIsOverriddenActive(id, true);
                        notesChildRef;
                        getOrderDetails();
                      }}
                      isOverridden={isEditPermission && item.IsCanOveride}
                      disabled={!isEditPermission}
                    />
                  )}
                </>
              );
            })}
        </Space>
      </Col>
    </Row>
  );
};

export default OrderFlagsContent;
