/* eslint-disable max-lines-per-function */
/* eslint-disable max-len */

import React, { useEffect, useState } from "react";
import { Col, Row, Space, Divider, Typography, Select, Spin } from "antd";
const { Text } = Typography;
import "./OrderSummary.scss";
import { OrderSummaryContentProps } from "interfaces/ClientInterface/IEditOrder";
import { DOLLAR, Minus, PLUS } from "config/constant";
import { notificationController } from "components/Common/Notification/Notification";
import {
  IAccountListItem,
  IOrderTypeListItem,
  IUserListItem,
} from "interfaces/APIInterface/ICreateOrder";
import { getAccountList, getOrderTypeList, getUserList } from "api/services/createOrder.api";
import { translate } from "i18n";
import { useAppDispatch, useAppSelector } from "hooks/reduxHooks";
import { convertDateTime } from "utils";
import { getGlobalSetting } from "api/services/localStorage.service";
import { getAccountId } from "store/slices/editOrder.slice";

/**
 * @returns This is a React functional component named `OrderSummaryContent` that renders a
 * layout of order summary information, including order details, order status, and order total.
 * The first column displays dropdown menu to select values (`Order
 * Summary`, `Accounts`, and `User`).
 * The second column displays a order summary.
 * The third column displays the order total summary
 */
const OrderSummaryContent: React.FC<OrderSummaryContentProps> = ({
  orderDetails,
  orderData,
  setOrderData,
  editOrderError,
  setEditOrderError,
  orderCalculation,
  getOrderDetailsWithFlag,
  initialOrderTypeCode,
  initialAccountId,
  initialAssignedUserId,
}: OrderSummaryContentProps) => {
  const dispatch = useAppDispatch();
  const isEditPermission = useAppSelector((state) => state.permission.isEditPermission);
  const [loader, setLoader] = useState({
    storeLoading: false,
    accountLoading: false,
    userLoading: false,
    orderTypeLoading: false,
  });

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const [userListData, setUserListData] = useState<IUserListItem[]>([]);
  const [accountList, setAccountList] = useState<IAccountListItem[]>([]);
  const [userList, setUserList] = useState<IUserListItem[]>([]);
  const [searchTerm, setSearchTerm] = useState<string>("");
  const [orderTypeList, setOrderTypeList] = useState<IOrderTypeListItem[]>([]);
  const [callUserListOnceOnLoad, setCallUserListOnceOnLoad] = useState(true);
  const globalSettings = getGlobalSetting() ? JSON.parse(getGlobalSetting() || "") : {};
  const priceRoundOff = globalSettings?.PriceRoundOff;
  const defaultAmount = `${DOLLAR}` + 0?.toFixed(priceRoundOff);

  const setLoading = (key: string, loaderState: boolean) => {
    setLoader((prevState) => ({
      ...prevState,
      [key]: loaderState,
    }));
  };

  useEffect(() => {
    if (searchTerm) {
      searchTerm.length >= 3 && fetchUserList();
    } else {
      setUserList([]);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [searchTerm]);

  useEffect(() => {
    if (!orderData?.userId) {
      setSearchTerm("");
      setUserList([]);
    }
  }, [orderData?.userId]);

  //useEffect to load only Account that has associated with the selected user
  useEffect(() => {
    if (callUserListOnceOnLoad && orderData?.userName) {
      const searchTerm = orderData?.userName?.substring(0, 3);
      setSearchTerm(searchTerm);
      setCallUserListOnceOnLoad(false);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [callUserListOnceOnLoad, orderData?.userName]);

  const fetchOrderTypeList = () => {
    setLoading("orderTypeLoading", true);
    getOrderTypeList()
      .then((response) => {
        const data = response?.data?.OrderType;
        setOrderTypeList(data);
        setLoading("orderTypeLoading", false);
      })
      .catch((err) => {
        notificationController.error({ message: err.message, placement: "top" });
        setLoading("orderTypeLoading", false);
      });
  };

  const fetchAccountList = () => {
    const UserDetails = userList?.find((item: IUserListItem) => {
      return item?.UserId == orderData?.userId;
    });
    setLoading("accountLoading", true);
    getAccountList(orderData?.storeId?.toString(), UserDetails?.AccountId || "")
      .then((response) => {
        const data = response?.data?.Accounts;
        setAccountList(data);
        if (orderData?.userId) {
          if (UserDetails?.AccountId === null || undefined) {
            setAccountList([]);
          }
        }
        setLoading("accountLoading", false);
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
        setLoading("accountLoading", false);
      });
  };

  const fetchUserList = () => {
    setLoading("userLoading", true);
    getUserList(orderData?.storeId?.toString(), searchTerm, orderData?.accountId?.toString())
      .then((response) => {
        const data = response?.data?.Users;
        const associatedAccountId = data?.find((item: IUserListItem) => {
          return item?.UserId == orderData?.userId;
        });
        dispatch(getAccountId(associatedAccountId?.AccountId || ""));
        setUserListData(data);
        setUserList(data);
        setLoading("userLoading", false);
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
        setLoading("userLoading", false);
      });
  };
  const getSelectedUser = (value: number) => {
    const getFilteredUserData = userListData?.find((item: IUserListItem) => {
      return item.UserId == value;
    });
    dispatch(getAccountId(getFilteredUserData?.AccountId || ""));
  };

  return (
    <Row wrap justify={"space-evenly"} className="summary-container">
      <Col span={10} className="align-center">
        <Space direction="vertical" size={14}>
          <Row>
            <Col className="key-content d-flex align-center">
              <Text className="key-title">{translate("editOrder.storeTitle")}:</Text>
            </Col>
            <Col className="value-content">
              <Select
                showSearch
                allowClear
                value={orderData?.storeName || undefined}
                className="dropdown-container"
                data-test-selector="drpStoreList"
                fieldNames={{ label: "Name", value: "Id" }}
                loading={loader?.storeLoading}
                disabled={true}
              />
            </Col>
          </Row>
          <Row>
            <Col className="key-content d-flex align-center">
              <Text className="key-title">{translate("editOrder.accountTitle")}:</Text>
            </Col>
            <Col className="value-content">
              <Select
                showSearch
                options={accountList}
                placeholder={translate("editOrder.selectAccountPlaceholder")}
                allowClear
                value={orderData?.accountName || undefined}
                className="dropdown-container"
                data-test-selector="drpAccountList"
                fieldNames={{ label: "Name", value: "AccountId" }}
                onClick={fetchAccountList}
                loading={loader?.accountLoading}
                // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-explicit-any
                onChange={(selectedText: string, option: any): void =>
                  setOrderData((prevState) => ({
                    ...prevState,
                    accountId: option?.AccountId,
                    accountName: option?.Name,
                    isAccountTypeChanged: initialAccountId != option?.AccountId ? true : false,
                  }))
                }
                // eslint-disable-next-line
                filterOption={(input, option?: any) =>
                  option?.Name?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
                }
                disabled={!isEditPermission}
                notFoundContent={null}
              />
            </Col>
          </Row>
          <Row>
            <Col className="key-content d-flex align-center">
              <Text className="key-title required">{translate("editOrder.userTitle")}:</Text>
            </Col>
            <Col className="value-content">
              <Select
                showSearch
                options={userList}
                placeholder={translate("editOrder.selectUserPlaceholder")}
                allowClear
                value={orderData?.userName || undefined}
                className="dropdown-container"
                data-test-selector="drpUserList"
                fieldNames={{ label: "UserName", value: "UserId" }}
                // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-explicit-any
                onChange={(selectedText: string, option: any): void => {
                  setOrderData((prevState) => ({
                    ...prevState,
                    userId: option?.UserId,
                    userName: option?.UserName,
                    isAssignedUserChanged: initialAssignedUserId != option?.UserId ? true : false,
                  }));
                  getSelectedUser(option?.UserId);
                  setEditOrderError((prevState) => ({
                    ...prevState,
                    userError: "",
                  }));
                }}
                loading={loader?.userLoading}
                notFoundContent={loader?.userLoading ? <Spin size="small" /> : null}
                filterOption={false}
                onSearch={(searchTerm: string) => setSearchTerm(searchTerm)}
                disabled={!isEditPermission}
              />
              <div className="error d-flex">{editOrderError && editOrderError?.userError}</div>
            </Col>
          </Row>
          <Row>
            <Col className="key-content d-flex align-center">
              <Text className="key-title required">{translate("editOrder.orderTypeTitle")}:</Text>
            </Col>
            <Col className="value-content">
              <Select
                showSearch
                options={orderTypeList}
                allowClear
                value={orderData?.orderTypeName || undefined}
                // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-explicit-any
                onChange={(selectedText: string, option: any): void => {
                  setOrderData((prevState) => ({
                    ...prevState,
                    orderTypeId: option?.CPOrderTypeId,
                    orderTypeName: option?.TypeName,
                    orderTypeCode: option?.TypeCode,
                    isOrderTypeChanged: initialOrderTypeCode != option?.TypeCode ? true : false,
                  }));
                  setEditOrderError((prevState) => ({
                    ...prevState,
                    orderTypeError: "",
                  }));
                  getOrderDetailsWithFlag && getOrderDetailsWithFlag();
                }}
                onClick={fetchOrderTypeList}
                className="dropdown-container"
                data-test-selector="drpOrderTypeList"
                fieldNames={{ label: "TypeName", value: "CPOrderTypeId" }}
                loading={loader?.orderTypeLoading}
                notFoundContent={loader?.orderTypeLoading ? <Spin size="small" /> : null}
                disabled={!isEditPermission}
              />
              <div className="error d-flex">{editOrderError && editOrderError?.orderTypeError}</div>
            </Col>
          </Row>
        </Space>
      </Col>
      <Col span={8} className="align-center">
        <Space direction="vertical" size={6} className="w-100">
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.orderCreatedTitle")}:</Text>
            </Col>
            <Col className="value-content">
              {orderDetails?.OrderDate && convertDateTime(orderDetails?.OrderDate)}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.inHandsDateTitle")}:</Text>
            </Col>
            <Col className="value-content">
              {orderDetails?.ShipDate && convertDateTime(orderDetails?.ShipDate)}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.finalizedTitle")}:</Text>
            </Col>
            <Col className="value-content">
              {orderDetails?.FinalizedDate && convertDateTime(orderDetails?.FinalizedDate)}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.createdByTitle")}:</Text>
            </Col>
            <Col className="value-content">{orderDetails?.CreatedByFullName}</Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.originTitle")}:</Text>
            </Col>
            <Col className="value-content">{orderDetails?.OrderOrigin}</Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.lastUpdatedTitle")}:</Text>
            </Col>
            <Col className="value-content">
              {orderDetails?.ModifiedDate != null && convertDateTime(orderDetails?.ModifiedDate)}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.lastUpdatedByTitle")}:</Text>
            </Col>
            <Col className="value-content">{orderDetails?.ModifiedByFullName}</Col>
          </Row>
        </Space>
      </Col>
      <Col span={6}>
        <Space direction="vertical" size={6} className="w-100">
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.subTotalTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.SubTotal === "number"
                ? DOLLAR + orderCalculation?.SubTotal?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.discountTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.Discount === "number"
                ? "-" + DOLLAR + orderCalculation?.Discount?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.taxTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.TaxCost === "number"
                ? PLUS + DOLLAR + orderCalculation?.TaxCost?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.handlingFeeTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.HandlingFee === "number"
                ? PLUS + DOLLAR + orderCalculation?.HandlingFee?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.shippingCostTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.ShippingCost === "number"
                ? PLUS + DOLLAR + orderCalculation?.ShippingCost?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          {orderCalculation && orderCalculation?.ShippingDiscount && orderCalculation?.ShippingDiscount !=0 ? (
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.shippingDisTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.ShippingDiscount === "number"
                ? Minus + DOLLAR + orderCalculation?.ShippingDiscount?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
        ): null}
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.csrDiscountTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.CSRDiscountAmount === "number" &&
              orderCalculation?.CSRDiscountApplied == true
                ? "-" + DOLLAR + orderCalculation?.CSRDiscountAmount?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title">{translate("editOrder.voucherAmountTitle")}</Text>
            </Col>
            <Col className="d-flex place-content-end value-content">
              {typeof orderCalculation?.VoucherAmount === "number"
                ? "-" + DOLLAR + orderCalculation?.VoucherAmount?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
          <Row>
            <Divider className="divider-margin" />
          </Row>
          <Row justify="space-between" align={"middle"}>
            <Col className="key-content">
              <Text className="key-title font-blue ">{translate("editOrder.totalTitle")}: </Text>
            </Col>
            <Col className="font-blue key-title d-flex place-content-end value-content">
              {typeof orderCalculation?.Total === "number"
                ? DOLLAR + orderCalculation?.Total?.toFixed(priceRoundOff)
                : defaultAmount}
            </Col>
          </Row>
        </Space>
      </Col>
    </Row>
  );
};
export default OrderSummaryContent;
