import React, { useContext } from "react";
import { Space } from "antd";
import { Typography } from "antd";
import { IOrder, OrderSummaryHeaderProps } from "interfaces/ClientInterface/IEditOrder";
import { DOLLAR } from "config/constant";
import { translate } from "i18n";
import { AbilityContext } from "ability/can";
import { useAppSelector } from "hooks/reduxHooks";
import StatusDropdown from "components/Common/Status/StatusDropdown";
import StatusTag from "components/Common/Status/StatusTag";
import { getStatusDropDownStyle } from "components/Common/Status/Status.Constant";
import { notificationController } from "components/Common/Notification/Notification";
import { getGlobalSetting } from "api/services/localStorage.service";
import { PERMISSION_CODE, PERMISSION_SUBJECT } from "ability/utils";
const { Text } = Typography;

/**
 * @param {string} accounts - The name of account
 * @param {string} user - The name of user
 * @param {string} orderStatus - status of order
 * @param {string} total - total value of the order
 * @returns this functional component returns a JSX element that renders
 * header for order summary collapse.
 */
export default function OrderSummaryHeader({
  orderDetails,
  orderStatusList,
  setOrder,
  orderTotal,
}: OrderSummaryHeaderProps) {
  const ability = useContext(AbilityContext);
  const canSetAnyOrderStatus = ability.can(
    PERMISSION_CODE.SET_ANY_ORDER_STATUS,
    PERMISSION_SUBJECT.ORDERS
  );
  const canCancelOrders = ability.can(PERMISSION_CODE.CANCEL_ORDERS, PERMISSION_SUBJECT.ORDERS);
  const isEditPermission = useAppSelector((state) => state.permission.isEditPermission);
  const globalSettings = getGlobalSetting() ? JSON.parse(getGlobalSetting() || "") : {};
  const priceRoundOff = globalSettings?.PriceRoundOff;
  const defaultAmount = `${DOLLAR}` + 0?.toFixed(priceRoundOff);

  return (
    <Space align="start" size={20}>
      <Text className="m-0 accordion-heading"> {translate("editOrder.orderSummaryTitle")}</Text>
      <Text className="m-0">|</Text>
      <Text className="m-0">
        <span className="accordion-heading">{translate("editOrder.accountTitle")}: </span>{" "}
        {orderDetails?.AccountName == null ? "NA" : orderDetails?.AccountName}
      </Text>
      <Text className="m-0">
        <span className="accordion-heading">{translate("editOrder.userTitle")}: </span>{" "}
        {orderDetails?.FullName}
      </Text>
      <Space size={24}>
        <div className="accordion-heading">{translate("editOrder.orderStatusTitle")}:</div>
        {isEditPermission && canSetAnyOrderStatus ? (
          <StatusDropdown
            orderStatusList={orderStatusList}
            size="small"
            value={orderDetails?.OrderStateName || undefined}
            placeholder="Select"
            bordered={false}
            data-test-selector={"drpStatus"}
            onChange={(selectedText: string, option: IOrder): void => {
              /**
               * if the user doesn't have Cancel Order Permission then restrict
               * the user to select this option and trigger notification message
               */
              if (option?.OrderStateName == "CANCELED" && !canCancelOrders) {
                notificationController.error({
                  message: translate("login.invalidLoginPermission"),
                  placement: "top",
                });
                return;
              }
              setOrder((prevState) => ({
                ...prevState,
                OrderStateName: option.OrderStateName,
                OmsOrderStateId: option.OmsOrderStateId,
              }));
            }}
            selectedValueKey="OrderStateName"
          />
        ) : (
          <StatusTag
            text={orderDetails?.OrderStateName}
            tagStyle={getStatusDropDownStyle(
              orderStatusList,
              orderDetails?.OrderStateName,
              "OrderStateName"
            )}
            dataTestSelector="tagOrderStatus"
          />
        )}
      </Space>
      <Space size={16}>
        <div className="accordion-heading">{translate("editOrder.orderTotalTitle")}:</div>
        <div>
          {typeof orderTotal === "number"
            ? DOLLAR + orderTotal?.toFixed(priceRoundOff)
            : defaultAmount}
        </div>
      </Space>
    </Space>
  );
}
