/* eslint-disable max-len */
import React, { useContext } from "react";
import { Button, Dropdown, MenuProps, Row, Space, Switch, Typography } from "antd";
import { PlusOutlined, CaretDownOutlined, MinusOutlined } from "@ant-design/icons";
import { IProductListHeader } from "interfaces/ClientInterface/IEditOrder";
import { useNavigate, useParams } from "react-router-dom";
import { ROUTES_PATH } from "routes/Routes.Constant";
import { translate } from "i18n";
import { DELETE_TYPES } from "./EditOrder.Constant";
import { checkPermissionToShow } from "utils";
import { useAppSelector } from "hooks/reduxHooks";
import { PureAbility } from "@casl/ability";
import { AbilityContext } from "ability/can";
import { PERMISSION_CODE, PERMISSION_SUBJECT } from "ability/utils";

const { Text } = Typography;

/**
 * @returns This React functional component returns a JSX element that renders
 * a header for a product list Table. The header contains buttons for adding
 * products, bulk adding, and clearing all products. It also includes a
 * dropdown menu for tools and a switch for toggling between
 * expanded and compact views.
 */

export const ProductListHeader: React.FC<IProductListHeader> = ({
  IsExpandable,
  setIsExpandable,
  ProductToolshandler,
  setIsQuickAddModalVisible,
  productList,
}: IProductListHeader) => {
  const navigate = useNavigate();
  const isEditPermission = useAppSelector((state) => state.permission.isEditPermission);
  const { orderId } = useParams<Record<string, string | undefined>>();
  
  const ability : PureAbility<[string, string]> = useContext(AbilityContext);
  const canChangePrices = ability.can(PERMISSION_CODE.CHANGE_PRICES, PERMISSION_SUBJECT.ORDERS);
  /**
   * This function is used to navigate to the add products to the order page.
   */
  const addProductToOrder = () => {
    navigate(`/${ROUTES_PATH.ADD_PRODUCTS}/${orderId}`);
  };

  const items: MenuProps["items"] = [
    {
      label: <div onClick={() => ProductToolshandler(DELETE_TYPES.DELETE_MULTIPLE)}>Delete</div>,
      key: "3",
      className: !isEditPermission ? "display-none" : "",
    },
    {
      label: <div onClick={() => ProductToolshandler("resetPrice")}>Reset Price</div>,
      key: "4",
      className: !isEditPermission || !canChangePrices ? "display-none" : "",
    },
    {
      label: (
        <div onClick={() => ProductToolshandler("SendCartLine")}>Send Cart Line Item Mail</div>
      ),
      key: "5",
    },
  ];

  return (
    <Row justify={"space-between"} align={"middle"} className="pb-8">
      <Space size={10}>
        {checkPermissionToShow(
          isEditPermission,
          <Button
            size="small"
            type="primary"
            icon={<PlusOutlined />}
            data-test-selector="btnAddProduct"
            onClick={addProductToOrder}
          >
            {translate("editOrder.addProductBtn")}
          </Button>
        )}
        {checkPermissionToShow(
          isEditPermission,
          <Button
            size="small"
            type="text"
            icon={<PlusOutlined />}
            data-test-selector="btnBulkAdd"
            onClick={() => {
              setIsQuickAddModalVisible(true);
            }}
          >
            {translate("editOrder.bulkAddBtn")}
          </Button>
        )}
        {checkPermissionToShow(
          isEditPermission,
          <Button
            onClick={() => ProductToolshandler(DELETE_TYPES.CLEAR_ALL)}
            size="small"
            type="text"
            icon={<MinusOutlined />}
            data-test-selector="btnClearAll"
            style={{ display: !productList?.length ? "none" : "inline-block" }}
          >
            {translate("editOrder.clearAllBtn")}
          </Button>
        )}
      </Space>
      <Space size={20}>
        <Dropdown
          menu={{ items }}
          trigger={["click"]}
          getPopupContainer={(triggerNode: HTMLElement) => triggerNode.parentNode as HTMLElement}
        >
          <Space>
            {translate("editOrder.toolsTitle")}
            <CaretDownOutlined />
          </Space>
        </Dropdown>
        <Space align="center" className="d-flex align-center">
          <Text>{translate("editOrder.expandedViewTitle")}</Text>
          <Switch
            size="small"
            checked={IsExpandable}
            onChange={(checked) => setIsExpandable(checked)}
            className="switch-top"
          />
        </Space>
      </Space>
    </Row>
  );
};
