import React from "react";
import { Col, Row, Space, Typography } from "antd";
import { CheckBoxComponent } from "components/Common/Checkbox/Checkbox";
import { CheckboxChangeEvent } from "antd/es/checkbox";
import { StopFilled } from "@ant-design/icons";
import { IAddUpdateOrderFlag, IOrderFlag } from "interfaces/ClientInterface/IOrderFlag";
import { deleteOrderFlag, updateOrderFlag } from "api/services/editOrder.api";
import { notificationController } from "components/Common/Notification/Notification";
import { readUserId } from "api/services/localStorage.service";
import { IProductFlagProps } from "interfaces/ClientInterface/IProductFlag";
import { REMOVE_FLAG_SUCCESS, UPDATE_FLAG_SUCCESS } from "./ProductFlags.Constant";
import { translate } from "i18n";
const { Text } = Typography;

/**
 * @returns This React functional component returns a JSX element that renders
 * multiple check-boxes to check between different product flags
 */
const ProductFlagsContent: React.FC<IProductFlagProps> = ({
  productFlags,
  getUnAssociatedProductFlags,
  fetchOrderProductDetails,
}: IProductFlagProps) => {
  /**
   * This function is called on changing the check-box selection of the product flags
   */
  const onChangeOrderProductFlag = (isChecked: boolean, orderFlagId: string) => {
    const updateOrderProductFlagPayload: IAddUpdateOrderFlag = {      
      UserId: readUserId(),
      OrderFlagId: orderFlagId,
      IsChecked: isChecked,
    };
    triggerUpdate(updateOrderProductFlagPayload);
  };

  /**
   * This function is called to delete or un-associate the flag from the product
   */
  const onDeleteOrderProductFlag = (orderFlagId: string) => {
    deleteOrderFlag(orderFlagId)
      .then((response) => {
        if (response.IsSuccess) 
        {
          notificationController.success({ message: REMOVE_FLAG_SUCCESS });
          fetchOrderProductDetails();
          getUnAssociatedProductFlags && getUnAssociatedProductFlags(false);
        }
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
      });
  };

  /**
   * This function is called on checkbox overridden selection
   */
  const onChangeIsOverriddenActive = async (orderFlagId: string, isActive: boolean) => {
    const updateOrderProductFlagPayload: IAddUpdateOrderFlag = {
      UserId: readUserId(),
      OrderFlagId: orderFlagId,
      IsActiveOverridden: isActive,
      IsCallForOverrideUserGroup: true,
    };

    if (await triggerUpdate(updateOrderProductFlagPayload, true)) {
      notificationController.success({ message: UPDATE_FLAG_SUCCESS });
      fetchOrderProductDetails();
    }
  };

  /**
   * This function is called to update the product flag
   */
  const triggerUpdate = (
    updateOrderProductFlagPayload: IAddUpdateOrderFlag,
    showMessage?: boolean
  ) => {
    return updateOrderFlag(updateOrderProductFlagPayload)
      .then((response) => {
        if (response?.OrderFlagStatus?.IsSuccess && showMessage) {
          notificationController.success({
            message: translate("editProduct.flagStatusSuccessMessage"),
          });
          return true;
        } else if (
          updateOrderProductFlagPayload.IsCallForOverrideUserGroup &&
          !response?.OrderFlagStatus?.IsSuccess &&
          !response?.OrderFlagStatus?.IsAllowFlagOverride
        ) {
          notificationController.error({
            message: translate("editProduct.loggedInUserNotPermittedToOverrideTheFlag"),
          });
        } else if (
          updateOrderProductFlagPayload.IsCallForOverrideUserGroup &&
          response?.OrderFlagStatus?.IsSuccess &&
          response?.OrderFlagStatus?.IsAllowFlagOverride
        ) {
          notificationController.success({
            message: translate("editProduct.flagStatusSuccessMessage"),
          });
        }
        fetchOrderProductDetails();
        return false;
      })
      .catch((err) => {
        notificationController.error({ message: err.message });
        return false;
      });
  };

  return (
    <Row justify={"start"} className="summary-container product-flags">
      <Col span={24}>
        <Space direction="vertical" size={[8,20]} className="product-flags-items">
          {productFlags &&
            productFlags?.map((item: IOrderFlag, i: number) => {
              return (
                <>
                  {item.IsActiveOverridden ? (
                    <Space size={10} key={i} id={item.OrderFlagId}>
                      <StopFilled
                        style={{ color: "#FF4D4F" }}
                        onClick={() => onChangeIsOverriddenActive(item.OrderFlagId, false)}
                      />
                      <Text>{item.NameOverridden}</Text>
                    </Space>
                  ) : (
                    <CheckBoxComponent
                      key={i}
                      id={item.OrderFlagId}
                      name={item.IsChecked ? item.NameChecked : item.NameUnchecked}
                      checked={item.IsChecked}
                      closable={true}
                      dataTestSelector={"chkProductFlag"}
                      onChangeHandle={(e: CheckboxChangeEvent) => {
                        onChangeOrderProductFlag(e.target.checked, item.OrderFlagId);
                        notificationController.success({
                          message: translate("editProduct.flagStatusSuccessMessage"),
                        });
                      }}
                      onCancelHandle={(id: string) => onDeleteOrderProductFlag(id)}
                      onOverriddenHandle={(id: string) =>
                        !item.IsChecked && onChangeIsOverriddenActive(id, true)
                      }
                      isOverridden={item.IsCanOveride}
                    />
                  )}
                </>
              );
            })}
        </Space>
      </Col>
    </Row>
  );
};

export default ProductFlagsContent;
