import React, { useState } from "react";
import { Button, Card, Col, Form, Input, Image, Row, Space, Typography } from "antd";
import HeaderRibbon from "../Common/HeaderRibbon/HeaderRibbon";
import { Footer } from "../Common/Footer/Footer";
import { useNavigate } from "react-router-dom";
import { notificationController } from "components/Common/Notification/Notification";
import axios from "axios";
import { BASE_URL, FORGOT_PASSWORD_API } from "config/constant";
import { isUsernameValid } from "utils/validation";
import { ROUTES_PATH } from "routes/Routes.Constant";
import { translate } from "i18n";
import { IForgotPassword } from "interfaces/ClientInterface/IForgotPassword";
import "./ForgotPassword.scss"; 

// eslint-disable-next-line max-lines-per-function
const ForgotPassword: React.FC = () => {
  const { Title } = Typography;
  const [isLoading, setLoading] = useState(false);
  const navigate = useNavigate();

  const handleCancel = () => {
    navigate(ROUTES_PATH.LOGOUT);
  };

  const handleSubmit = (forgotPassword: IForgotPassword) => {
    setLoading(true);
    forgotPassword.baseURL = BASE_URL;
    forgotPassword.user = {
      email: forgotPassword.username,
      username: forgotPassword.username,
    };
    axios
      .post<IForgotPassword>(FORGOT_PASSWORD_API, { ...forgotPassword })
      .then((response) => {
        response?.status === 200 &&
          notificationController.success({
            message: translate("forgotPassword.linkSentMessage"),
            placement: "top",
          });
        setLoading(false);
      })
      .catch((err) => {
        if (err?.response?.status === 500) {
          notificationController.error({
            message: translate("forgotPassword.userNameValidateMsg"),
            placement: "top",
          });
        } else {
          notificationController.error({ message: err.message, placement: "top" });
        }
        setLoading(false);
      });
  };

  return (
    <Space direction="vertical" size={99} style={{ display: "flex" }}>
      <HeaderRibbon ribbonHeight={30} />
      <Row justify={"center"} align={"middle"} className="forgot-container">
        <Col span={8} md={{ span: 12 }} xl={{ span: 8 }} xs={{ span: 18 }} lg={{ span: 12 }}>
          <Card bordered={true}>
            <Space direction="vertical" size="middle" style={{ display: "flex" }}>
              <Row justify={"center"}>
                <Col span={12}>
                  <Image className="login-card" src="/logo-massive.png" preview={false} />
                </Col>
              </Row>
              <Row justify={"center"}>
                <Col span={24} className="text-center">
                  <Title level={4} className="m-0 forgot-header">
                    {translate("forgotPassword.title")}
                  </Title>
                </Col>
              </Row>
              <Row justify={"center"}>
                <Col span={24}>
                  <Form
                    layout="vertical"
                    name="basic"
                    initialValues={{ remember: true }}
                    autoComplete="off"
                    onFinish={handleSubmit}
                    className="forgot-password-container"
                    data-test-selector="frmForgotPassword"
                  >
                    <Space direction="vertical" size="small" style={{ display: "flex" }}>
                      <Form.Item
                        name="username"
                        label={translate("forgotPassword.username")}
                        rules={[
                          () => ({
                            validator(_, value) {
                              if (!value) {
                                return Promise.reject(
                                  new Error(translate("forgotPassword.userNameRequired"))
                                );
                              }
                              if (value.trim() !== "" && !isUsernameValid(value)) {
                                return Promise.reject(
                                  new Error(translate("forgotPassword.userNameValid"))
                                );
                              }
                              return Promise.resolve();
                            },
                          }),
                        ]}
                        validateTrigger="onBlur"
                      >
                        <Input
                          placeholder={translate("forgotPassword.username")}
                          data-test-selector="txtForgotPasswordUsername"
                        />
                      </Form.Item>
                      <Form.Item>
                        <Row justify={"end"}>
                          <Col
                            span={24}
                            md={{ span: 24 }}
                            xl={{ span: 24 }}
                            xs={{ span: 24 }}
                            lg={{ span: 24 }}
                            className="text-right"
                          >
                            <Button
                              htmlType="button"
                              className="cancel-button mr-15"
                              data-test-selector="btnCancel"
                              onClick={handleCancel}
                            >
                              {translate("forgotPassword.cancel")}
                            </Button>
                            <Button
                              type="primary"
                              htmlType="submit"
                              className="login-form-button"
                              data-test-selector="btnResetPassword"
                              loading={isLoading}
                            >
                              {translate("forgotPassword.resetBtnText")}
                            </Button>
                          </Col>
                        </Row>
                      </Form.Item>
                    </Space>
                  </Form>
                </Col>
              </Row>
            </Space>
          </Card>
        </Col>
      </Row>
      <Footer />
    </Space>
  );
};

export default ForgotPassword;