/* eslint-disable max-lines-per-function */
import React, { useContext, useState } from "react";
import { Button, Divider, Popover, Select, Space, Typography } from "antd";
import { AssignToUserModalProps } from "interfaces/ClientInterface/IAssignedTo";
import { CloseOutlined } from "@ant-design/icons";
import { CheckBoxComponent } from "components/Common/Checkbox/Checkbox";
import { CheckboxChangeEvent } from "antd/es/checkbox";
import { IAssignUnassign } from "interfaces/APIInterface/IAssignTo";
import { readUserId,getUserDetails } from "api/services/localStorage.service";
import { notificationController } from "components/Common/Notification/Notification";
import { updateOrderOwner } from "api/services/editOrder.api";
import { translate } from "i18n";
import { PureAbility } from "@casl/ability";
import { checkAssignToMePermission, checkAssignToMyTeamPermission } from "./Order.Constant";
import { AbilityContext } from "ability/can";

const { Text } = Typography;

export const AssignToUserModal = ({
  open,
  setOpen,
  children,
  calledFromTools = false,
  refetchList,
  orderId,
  myTeam,
  selectedRowKeys,
  setSelectedRowKeys,
  IsDelegateToMe,
}: AssignToUserModalProps) => {
  const ability: PureAbility<[string, string]> = useContext(AbilityContext);
  const canAssignToMyTeamPermission = checkAssignToMyTeamPermission(ability);
  const canAssignToMePermission = checkAssignToMePermission(ability);
  const [isMyself, setIsMyself] = useState(false as boolean);
  const [userId, setUserId] = useState(0 as number);
  const [user, setUser] = useState("");
  // eslint-disable-next-line no-unused-vars
  const [isInputEmpty, setIsInputEmpty] = useState(false);
  const orderIds = calledFromTools ? selectedRowKeys?.join(",") : orderId;
  const assignTo = isMyself && userId ? userId : user;

  const hide = () => {
    clearFields();
    setOpen(false);
  };

  const clearFields = () => {
    setUser("");
    setIsMyself(false);
    //To clear validation message.
    setIsInputEmpty(false);
  };

  const onAssignUnassign = () => {
    if (!calledFromTools) {
      if (!user && !isMyself) {
        setIsInputEmpty(true);
        notificationController.error({
          message: translate("assignedTo.inputRequired"),
          placement: "top",
        });
        return;
      }
    } else if (!user) {
      setIsInputEmpty(true);
      notificationController.error({
        message: translate("assignedTo.inputRequired"),
        placement: "top",
      });
      return;
    }

    const assignUnassignPayload: IAssignUnassign = {
      OrderIds: orderIds,
      AssignTo: Number(assignTo),
      UserId: readUserId(),
    };

    updateOrderOwner(assignUnassignPayload)
      .then((resp) => {
        if (resp[0].OrderDetailJson) {
          notificationController.error({
            message: translate("manageOrder.customMessage"),
            placement: "top",
          });
        } else {
          notificationController.success({
            message: translate("assignedTo.ownerAssignedSuccessMessage"),
            placement: "top",
          });
        }
        setOpen(false);
        setUser("");
        setIsMyself(false);
        refetchList && refetchList();
        setSelectedRowKeys([]);
      })
      .catch(() => {
        notificationController.error({ message: translate("assignedTo.assignOwnerErrorMessage") });
      });
  };

  return (
    <Popover
      transitionName=""
      overlayClassName={isInputEmpty ? "error-popover-wrapper" : ""}
      align={calledFromTools ? { offset: [110, 0] } : {}}
      title={
        <Space direction="vertical" className="d-flex">
          <Space direction="horizontal" className="d-flex justify-between">
            <Text>{translate("assignedTo.titleAssignTo")}</Text>
            <CloseOutlined onClick={hide} />
          </Space>
          <Divider className="m-0" />
        </Space>
      }
      content={
        <Space size={8} direction="vertical">
          {!calledFromTools && (canAssignToMePermission || IsDelegateToMe) && (
            <CheckBoxComponent
              name={"Myself"}
              checked={isMyself}
              closable={false}
              dataTestSelector={"chkMyself"}
              onChangeHandle={(e: CheckboxChangeEvent) => {
                const filterUserId = myTeam.filter((item) =>
                // eslint-disable-next-line
                //@ts-ignore
                item.UserName === JSON.parse(getUserDetails()).UserName ? item.UserId : ""
                );
                setUserId(filterUserId[0].UserId);
                setIsMyself(e.target.checked);
                setIsInputEmpty(false);
              }}
            />
          )}
          {canAssignToMyTeamPermission && (
            <Select
              placeholder={translate("assignedTo.placeholderSelectUser")}
              options={myTeam}
              showSearch={true}
              data-test-selector="drpUser"
              onChange={(value) => {
                setUser(value);
                setIsInputEmpty(false);
              }}
              value={user || undefined}
              allowClear={true}
              style={{ width: 180 }}
              listHeight={128}
              fieldNames={{ label: "UserName", value: "UserId" }}
              disabled={isMyself}
              // eslint-disable-next-line @typescript-eslint/no-explicit-any
              filterOption={(input, option?: any) =>
                option?.UserName?.toLowerCase()?.indexOf(input?.toLowerCase()) >= 0
              }
            />
          )}
          <Button type="primary" data-test-selector={"btnSave"} onClick={onAssignUnassign}>
            {translate("assignedTo.btnSave")}
          </Button>
        </Space>
      }
      trigger="click"
      open={open}
      {...(calledFromTools ? { onOpenChange: (newOpen: boolean) => setOpen(newOpen) } : {})}
      placement="rightTop"
    >
      {calledFromTools ? <span>{children}</span> : <div className="assigned-title">{children}</div>}
    </Popover>
  );
};
