/* eslint-disable max-len */
import React, { useEffect, useState } from "react";
import { Button, Card, Col, Form, Input, Image, Row, Space, Typography } from "antd";
import HeaderRibbon from "../Common/HeaderRibbon/HeaderRibbon";
import { notificationController } from "components/Common/Notification/Notification";
import { Footer } from "../Common/Footer/Footer";
import { PORTAL_ID, RESET_PASSWORD_API, RESET_PASSWORD_VERIFY_API } from "config/constant";
import axios from "axios";
import { useLocation, useNavigate } from "react-router-dom";
import { isPasswordMatched, isPasswordStrength } from "utils/validation";
import { ROUTES_PATH } from "routes/Routes.Constant";
import { translate } from "i18n";
import { IResetPassword } from "interfaces/ClientInterface/IForgotPassword";
import "../ForgotPassword/ForgotPassword.scss";

// eslint-disable-next-line max-lines-per-function
export default function ResetPassword() {
  const { Title } = Typography;
  const [isLoading, setLoading] = useState(false);
  const [isDisabled, setDisabled] = useState(false);
  const navigate = useNavigate();
  const search = useLocation().search;
  const ERROR_MESSAGE = {
    Continue: 2001,
    LinkExpired: 2002,
    TokenMismatch: 2003,
    NoRecord: 2004,
  };

  useEffect(() => {
    const resetPassword: IResetPassword = {
      user: {
        username: new URLSearchParams(search).get("userName"),
        passwordToken: new URLSearchParams(search).get("passwordToken"),
      },
      portalId: PORTAL_ID,
    };
    axios
      .post<IResetPassword>(RESET_PASSWORD_VERIFY_API, { ...resetPassword })
      .then((response) => {
        const errorCode = response?.data?.ErrorCode;
        switch (errorCode) {
          case ERROR_MESSAGE.Continue:
            break;
          case ERROR_MESSAGE.LinkExpired || ERROR_MESSAGE.TokenMismatch:
            notificationController.error({
              message: translate("resetPassword.resetLinkExpiredErrorMessage"),
              placement: "top",
            });
            setDisabled(true);
            break;
          case ERROR_MESSAGE.NoRecord:
          default:
            navigate(ROUTES_PATH.FORGOT_PASSWORD);
        }
      })
      .catch((err) => {
        notificationController.error({
          message:
            err?.response?.status === 500
              ? translate("resetPassword.wrongUsernameErrorMessage")
              : err.message,
          placement: "top",
        });
      });
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [window.location.pathname]);

  const handleSubmit = (resetPassword: IResetPassword) => {
    setLoading(true);
    resetPassword.portalId = PORTAL_ID;
    resetPassword.user = {
      username: new URLSearchParams(search).get("userName"),
      newPassword: resetPassword.newPassword,
      passwordToken: new URLSearchParams(search).get("passwordToken"),
    };
    axios
      .post<IResetPassword>(RESET_PASSWORD_API, { ...resetPassword })
      .then((response) => {
        if (response?.status === 200) {
          setLoading(false);
          notificationController.success({
            message: translate("resetPassword.passwordResetSuccessMessage"),
          });
          setTimeout(() => {
            navigate(ROUTES_PATH.LOGOUT);
          }, 4000);
        }
      })
      .catch((err) => {
        notificationController.error({
          message:
            err?.response?.status === 500
              ? translate("resetPassword.passwordMatchedPreviousErrorMessage")
              : err.message,
          placement: "top",
        });
        setLoading(false);
      });
  };

  return (
    <Space direction="vertical" size={99} style={{ display: "flex" }}>
      <HeaderRibbon ribbonHeight={30} />
      <Row justify={"center"} align={"middle"} className="forgot-container">
        <Col span={8} md={{ span: 12 }} xl={{ span: 8 }} xs={{ span: 18 }} lg={{ span: 12 }}>
          <Card bordered={true}>
            <Space direction="vertical" size="middle" style={{ display: "flex" }}>
              <Row justify={"center"}>
                <Col span={12}>
                  <Image className="login-card" src="/logo-massive.png" preview={false} />
                </Col>
              </Row>
              <Row justify={"center"}>
                <Col span={24} className="text-center">
                  <Title level={4} className="m-0 forgot-header">
                    {translate("resetPassword.titleResetPassword")}
                  </Title>
                </Col>
              </Row>
              <Row justify={"center"}>
                <Col span={24}>
                  <Form
                    layout="vertical"
                    name="basic"
                    initialValues={{ remember: true }}
                    autoComplete="off"
                    onFinish={handleSubmit}
                    data-test-selector="frmResetPassword"
                  >
                    <Space direction="vertical" size="small" style={{ display: "flex" }}>
                      <Form.Item
                        name="newPassword"
                        label={translate("resetPassword.labelNewPassword")}
                        rules={[
                          () => ({
                            validator(_, value) {
                              if (!value) {
                                return Promise.reject(
                                  new Error(translate("resetPassword.newPasswordRequiredMessage"))
                                );
                              }
                              if (value.trim() !== "" && !isPasswordStrength(value)) {
                                return Promise.reject(
                                  new Error(
                                    translate("resetPassword.passwordConditionMatchErrorMessage")
                                  )
                                );
                              }
                              return Promise.resolve();
                            },
                          }),
                        ]}
                      >
                        <Input.Password
                          placeholder={translate("resetPassword.placeholderNewPassword")}
                          data-test-selector="pwdNewPassword"
                        />
                      </Form.Item>
                      <Form.Item
                        name="confirmPassword"
                        label={translate("resetPassword.labelConfirmNewPassword")}
                        dependencies={["newPassword"]}
                        rules={[
                          ({ getFieldValue }) => ({
                            validator(_, value) {
                              if (!value) {
                                return Promise.reject(
                                  new Error(translate("resetPassword.reEnterPasswordErrorMessage"))
                                );
                              }
                              if (
                                value.trim() !== "" &&
                                !isPasswordMatched(getFieldValue("newPassword"), value)
                              ) {
                                return Promise.reject(
                                  new Error(
                                    translate("resetPassword.passwordDoNotMatchErrorMessage")
                                  )
                                );
                              }
                              return Promise.resolve();
                            },
                          }),
                        ]}
                      >
                        <Input.Password
                          placeholder={translate("resetPassword.labelConfirmNewPassword")}
                          data-test-selector="pwdConfirmPassword"
                        />
                      </Form.Item>
                      <Form.Item>
                        <Row justify={"end"}>
                          <Col
                            span={24}
                            md={{ span: 24 }}
                            xl={{ span: 24 }}
                            xs={{ span: 24 }}
                            lg={{ span: 24 }}
                            className="text-right"
                          >
                            <Button
                              type="primary"
                              htmlType="submit"
                              className="login-form-button"
                              data-test-selector="btnResetPassword"
                              loading={isLoading}
                              disabled={isDisabled}
                            >
                              {translate("resetPassword.titleResetPassword")}
                            </Button>
                          </Col>
                        </Row>
                      </Form.Item>
                    </Space>
                  </Form>
                </Col>
              </Row>
            </Space>
          </Card>
        </Col>
      </Row>
      <Footer />
    </Space>
  );
}
