/* eslint-disable @typescript-eslint/no-unused-vars */
import { createAsyncThunk, createSlice } from "@reduxjs/toolkit";
import {
  ResetPasswordRequest,
  signUp,
  SignUpRequest,
  resetPassword,
  verifySecurityCode,
  SecurityCodePayload,
  NewPasswordData,
  setNewPassword,
  RefreshClient,
  setClientToken,
  login,
} from "api/auth.api";

import {
  clearLocalStorage,
  deleteToken,
  deleteUser,
  persistUserId,
  persistUserDetails,
  readToken,
  removeAllCookies,
  setAxiosHeader,
} from "api/services/localStorage.service";
import { COOKIES_EXPIRES, LOGIN_COOKIE, REMEMBER_COOKIE } from "config/constant";
import dayjs from "dayjs";
import { ILogin } from "interfaces/ClientInterface/ILogin";
import Cookies from "js-cookie";

export interface AuthSlice {
  token: string | null;
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  user: any | null;
  loading: boolean;
  error: string | null;
}

const LOGIN_ERROR_MESSAGE = "Invalid Username or Password.";

const initialState: AuthSlice = {
  token: readToken(),
  user: null,
  loading: false,
  error: "",
};

const createAuthPayload = (data: ILogin) => {
  return {
    user: {
      username: data.username,
      password: data.password,
    },
  };
};

const startOfDay = dayjs().startOf("day");
const endOfDay = dayjs().endOf("day");

const minutesInADay = endOfDay.diff(startOfDay, "minute");

export const doLogin = createAsyncThunk(
  "auth/login",
  // eslint-disable-next-line no-unused-vars
  async (loginPayload: ILogin, { dispatch }) => {
    try {
      const response = await login(createAuthPayload(loginPayload));
      if (loginPayload.remember) {
        Cookies.set(REMEMBER_COOKIE, "available", {
          expires: COOKIES_EXPIRES * minutesInADay,
        });
        Cookies.set(LOGIN_COOKIE, loginPayload.username, {
          expires: COOKIES_EXPIRES * minutesInADay,
        });
      } else {
        Cookies.remove(REMEMBER_COOKIE);
        localStorage.setItem("sessionActive", "available");
      }
      // setting the Znode userId in local storage and axios headers
      setAxiosHeader(response?.User?.UserId);
      persistUserId(response?.User?.UserId);
      // setting the user details in local storage
      const userDetails = {
        UserName: response?.User?.UserName,
        FirstName: response?.User?.FirstName,
        LastName: response?.User?.LastName,
      };
      persistUserDetails(userDetails);
      return response;
    } catch (error) {
      return Promise.reject(error);
    }
  }
);

export const doSetClient = createAsyncThunk(
  "auth/doSetClient",
  async (clientPayload: RefreshClient) =>
    setClientToken(clientPayload).then((res) => {
      return res;
    })
);

export const doSignUp = createAsyncThunk("auth/doSignUp", async (signUpPayload: SignUpRequest) =>
  signUp(signUpPayload)
);

export const doResetPassword = createAsyncThunk(
  "auth/doResetPassword",
  async (resetPassPayload: ResetPasswordRequest) => resetPassword(resetPassPayload)
);

export const doVerifySecurityCode = createAsyncThunk(
  "auth/doVerifySecurityCode",
  async (securityCodePayload: SecurityCodePayload) => verifySecurityCode(securityCodePayload)
);

export const doSetNewPassword = createAsyncThunk(
  "auth/doSetNewPassword",
  async (newPasswordData: NewPasswordData) => setNewPassword(newPasswordData)
);

export const doLogout = createAsyncThunk("auth/doLogout", () => {
  deleteToken();
  deleteUser();
  removeAllCookies();
  clearLocalStorage();
});

const authSlice = createSlice({
  name: "auth",
  initialState,
  reducers: {},
  extraReducers: (builder) => {
    builder.addCase(doLogin.pending, (state) => {
      state.loading = true;
    });
    builder.addCase(doLogin.fulfilled, (state, action) => {
      state.user = action.payload;
      state.loading = false;
    });
    builder.addCase(doLogin.rejected, (state) => {
      state.error = LOGIN_ERROR_MESSAGE;
      state.user = null;
      state.loading = true;
    });
    builder.addCase(doLogout.fulfilled, (state) => {
      state.user = null;
      state.loading = false;
    });
  },
});

export default authSlice.reducer;
